// **********************************************************************
// * \author Sven Havemann & Dominique Gunia,                           *
// *         Institute for Computer Graphics,                           *
// *         TU Braunschweig, Muehlenpfordtstrasse 23,                  *
// *         38106 Braunschweig, Germany                                *
// *         http://www.cg.cs.tu-bs.de      mailto:s.havemann@tu-bs.de  *
// * \date   25.04.2002                                                 *
// **********************************************************************


#ifndef DAVE_HH
#define DAVE_HH

#include "t_3dvect_new.hh"

class DaveCore;

/** class Dave
 *
 *  The coordinate system is as follows:
 *  The center of the world is in the center of Dave, 1.25m above the ground.
 *  - positive x is to your right
 *  - positive y is in front of you
 *  - positive z is upwards.
 */

class Dave
{
public:

   /** establishes connection right away */
   /** if you want to try at home, with no hardware, give a negative number
       for the udpSyncPort. This will create a Dave instance which acts like
       a server, has a center eye, and doesn't need any DAVE hardware or clients.
       This implies that head and mouse will not move, of course, and 
       synchronize() returns immediately. */ 

   Dave(int sizeOfSyncBuffer, int udpSyncPort = 1500);
   ~Dave(); 

   /** =================================== */
   /** QUERY FUNCTIONS                     */

   bool          isServer        (); 
   bool          isClient        (); 
   /* -1 if not on dave, 0 for daveserver, 1-8 for dave1-8 */
   int           getHost         (); 
   /* 0=server, 1=left, 2=front, 3=right, 4=floor */
   int           getScreen       (); 
   /* 0=server, 1=left, 2=right, 3=central (left==right) */
   int           getEye          ();
   /* for debugging set both eyes to the same value (no stereo effect) */
   void          setCenterEye    (); 
   /* sets near and front plane values, defaults are [0.01, 20.0] */
   void          setNearFarPlane (float near_plane, float far_plane);
   /* call this function once in your OpenGL setup (resize) function */
   void          initOpenGL      (); 
   /* call this on server to stop the program on server and all clients */
   void          stop            (); 
   /* this is false if Ctrl-C (SIGINT) has been hit on the server */
   bool          isStopped       ();
   
   /** =================================== */
   /** HEAD                                */

   /** Head position in world coordinates */
   t_3DVector    getHeadPosition (); 
   t_3DVector    getHeadEuler    (); 
   /** Head x,y,z vectors in world coordinates */
   t_3DVector    getHeadX        ();  /* right */
   t_3DVector    getHeadY        ();  /* look direction */
   t_3DVector    getHeadZ        ();  /* up */
   const float*  getHeadMatrix   ();  /* X,Y,Z as 4x4 OpenGL-Matrix */

   /** Make the head coordinate system the OpenGL coordinate system */
   void          headCSToOpenGL  ();

   /** =================================== */
   /** 6DOF MOUSE                          */

   /** Mouse position in world coordinates */
   t_3DVector    getMousePosition(); 
   t_3DVector    getMouseEuler   (); 

   /** Mouse x,y,z vectors in world coordinates */
   t_3DVector    getMouseX       ();  /* right */
   t_3DVector    getMouseY       ();  /* forward */
   t_3DVector    getMouseZ       ();  /* up */
   const float*  getMouseMatrix  ();  /* X,Y,Z as 4x4 OpenGL-Matrix */
   int           getMouseButton  ();  /* 1=left 2=right 3=middle button */   

   /** Make the mouse coordinate system the OpenGL coordinate system */
   void          mouseCSToOpenGL ();

   /** =================================== */
   /** CAVE PROJECTION                     */

   /** This function takes the head and eye into account and sets the current
       projection matrix accordingly. To be called at the beginning of your
       OpenGL output routine. You are completely free to use the modelview
       matrix as you like. */
   void          setProjectionGL (); 
   t_3DVector    getEyePosLeft   (); 
   t_3DVector    getEyePosRight  (); 
   
   /** =================================== */
   /** SYNCHRONIZATION                     */

   void          synchronize     (); 
   int           sizeOfSyncBuffer();
   char*         getSyncBuffer   (); 

   /** =================================== */
   /** FRAMETIME CONTROL for use on server */

   /** return 1.0/getMaxFrametime() if it is >0 */
   float         getFPS (); 
   /** return the maximal time (sec) the last frame took to render on a client */
   float         getMaxFrametime (); 
   /** return the id of the client which was slowest */
   int           getMaxTimeDave  (); 
   /** return the whole array of 8 frame rendering times (in secs) */
   const float*  getFrametimes   ();

private:
   DaveCore* v_dave; 
}; 

#endif
